#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 16 12:42:33 2020

@author: insauer
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from matplotlib.lines import Line2D

"""Script to reproduce main Figure 3!"""

fig3 = plt.figure(constrained_layout=True, figsize=(7.0, 9.9))
gs = fig3.add_gridspec(20, 14)
plt.subplots_adjust(wspace=-0., hspace=0)


DATA_ATTR_Full = pd.read_csv('../../data/reconstruction/attribution_MetaData_regions.csv')

DATA_ATTR = pd.read_csv('../../data/reconstruction/attribution_MetaData_subregions.csv')

DATA_FIT_Full= pd.read_csv('../../data/reconstruction/vulnerability_adjustment_MetaData_regions.csv')
DATA_FIT= pd.read_csv('../../data/reconstruction/vulnerability_adjustment_MetaData_subregions.csv')


region_names={'GLB': 'Global (GLB)',
              'NAM':'North America (NAM)',
              'EAS':'Eastern Asia (EAS)',
              'OCE':'Oceania (OCE)',
              'LAM':'Latin America (LAM)',
              'EUR':'Europe (EUR)',
              'CAS':'Central Asia (CAS)',
              'SSA':'South & Sub-Sahara Africa (SSA)',
              'SEA':'South & South-East Asia (SEA)', 
              'NAF':'North Africa & Middle East (NAF)'
              }

region_abs={'NAM':'NAM', 
          'LAM':'LAM', 
          'EUR':'WEU',
          'NAF':'NAR',
          'SSA':'SSA',
          'CAS':'CAS',
          'SEA':'SEA', 
          'EAS':'EAS', 
          'OCE':'OCE',
          'GLB': 'GLB'}

regions = list(region_names)

#order = [9,0,1,2,]
r =0

for i in range(4):
    for j in range(3):
        
        if i == 0 and (j ==0 or j ==2):
            continue

        f3_ax1 = fig3.add_subplot(gs[5*i:5*i+4,j*5:(j*5)+4])

        r_lin = DATA_FIT_Full.loc[DATA_FIT_Full['Region']==regions[r], 'R2_D_Full_D_Obs'].sum()     

        r_lin_pos = DATA_FIT.loc[DATA_FIT['Region']==regions[r]+'_pos', 'R2_D_Full_D_Obs'].sum()
        r_lin_neg = DATA_FIT.loc[DATA_FIT['Region']==regions[r]+'_neg', 'R2_D_Full_D_Obs'].sum()
        
        
        data_attr_reg = DATA_ATTR_Full[DATA_ATTR_Full['Region'] == regions[r]]
        data_attr_reg_pos = DATA_ATTR[DATA_ATTR['Region'] == regions[r]+'_pos']
        data_attr_reg_neg = DATA_ATTR[DATA_ATTR['Region'] == regions[r]+'_neg']
        
        h8 = data_attr_reg.loc[:,'C_1980_80'].sum()
        e8 = data_attr_reg.loc[:,'E_exposure'].sum()
        v8 = data_attr_reg.loc[:,'V_vulnerability'].sum()
        i8 = data_attr_reg.loc[:,'M_damage_modeled'].sum()
        n8 = data_attr_reg.loc[:,'N_damage_observed'].sum()
        
        h_pos8 =  data_attr_reg_pos.loc[:,'C_1980_80'].sum()
        e_pos8 = data_attr_reg_pos.loc[:,'E_exposure'].sum()
        v_pos8 = data_attr_reg_pos.loc[:,'V_vulnerability'].sum()
        i_pos8 = data_attr_reg_pos.loc[:,'M_damage_modeled'].sum()
        n_pos8 = data_attr_reg_pos.loc[:,'N_damage_observed'].sum()
    
        
        h_neg8 =  data_attr_reg_neg.loc[:,'C_1980_80'].sum()
        e_neg8 =  data_attr_reg_neg.loc[:,'E_exposure'].sum()
        v_neg8 =  data_attr_reg_neg.loc[:,'V_vulnerability'].sum()
        i_neg8 =  data_attr_reg_neg.loc[:,'M_damage_modeled'].sum()
        n_neg8 =  data_attr_reg_neg.loc[:,'N_damage_observed'].sum()
        

        
        h_sign = data_attr_reg.loc[:,'p_val_C_1980_80'].sum()
        h_sign_pos = data_attr_reg_pos.loc[:,'p_val_C_1980_80'].sum()
        h_sign_neg = data_attr_reg_neg.loc[:,'p_val_C_1980_80'].sum()

        x=[0,1,2,3,5,6,7,8,10,11,12,13]
        colour_code = ['darkblue','orange','brown','mediumpurple',
                       'darkblue','orange','brown','mediumpurple',
                       'darkblue','orange','brown','mediumpurple']
        y1= [h8,e8,v8,i8,h_pos8,e_pos8,v_pos8,i_pos8,h_neg8,e_neg8,v_neg8,i_neg8]


        
        if r_lin_pos > 0.2*100:
             f3_ax1.axvspan(4,9, facecolor='gainsboro')

            
        if r_lin > 0.2*100:
             f3_ax1.axvspan(-1,4, facecolor='gainsboro')

            
        if r_lin_neg > 0.2*100:
            
            f3_ax1.axvspan(9,14, facecolor='gainsboro')


        for a in range(12):
            
            if not (r==1 and a>3 and a<8) :
                
                if ((a == 0)and (h_sign > 0.1)) or ((a == 4) and (h_sign_pos > 0.1)) or ((a == 8)and (h_sign_neg > 0.1)):
                    
                    f3_ax1.bar(x[a],y1[a], color = "cornflowerblue")
                
                else:
                    
                    f3_ax1.bar(x[a],y1[a], color = colour_code[a], #fmt='o',
                            ecolor='black')#,mfc =colour_code[a])

        f3_ax1.set_xticks([])
        #f3_ax1.set_xticklabels(['','', '', '' ],fontsize =6.5)
        
        if r in [0,1,4,7]:
            f3_ax1.set_ylabel('Relative trend in % per year', fontsize=6, labelpad = 2.5)
            
        

        f3_ax1.set_xlim(-1.,14)
        
        if r <7:
            f3_ax1.set_ylim(-5, 12)
            f3_ax1.set_yticks([-5.,0, 5, 10 ])
            f3_ax1.set_yticklabels(['-5%','0%', '5%','10%'],fontsize =5)
        else:
            f3_ax1.set_ylim(-5, 33)
            f3_ax1.set_yticks([0, 10, 20, 30])
            f3_ax1.set_yticklabels(['0%','10%', '20%', '30%' ],fontsize =5)
        
        nes = [n8,n_pos8,n_neg8]
        
        xn = [4,9,14]
        
        xx= [3,8,13]
        
        for ns in range(3):
            if nes[ns]>0:
                lims = np.abs(f3_ax1.get_ylim()[0])/(f3_ax1.get_ylim()[1]-f3_ax1.get_ylim()[0]) + nes[ns]/(f3_ax1.get_ylim()[1]-f3_ax1.get_ylim()[0])
                zerlim = np.abs(f3_ax1.get_ylim()[0])/(f3_ax1.get_ylim()[1]-f3_ax1.get_ylim()[0])
                f3_ax1.axhline(y=nes[ns], xmin=(xn[ns]-0.4)/15, xmax=(xn[ns]+0.4)/15,linewidth=1, color='k')
                f3_ax1.axvline(x=xx[ns]-0.4, ymin=zerlim, ymax=lims,linewidth=1., color='k')
                f3_ax1.axvline(x=xx[ns]+0.4, ymin=zerlim, ymax=lims,linewidth=1., color='k')
            else:
                lims = np.abs(f3_ax1.get_ylim()[0])/(f3_ax1.get_ylim()[1]-f3_ax1.get_ylim()[0]) + nes[ns]/(f3_ax1.get_ylim()[1]-f3_ax1.get_ylim()[0])
                zerlim = np.abs(f3_ax1.get_ylim()[0])/(f3_ax1.get_ylim()[1]-f3_ax1.get_ylim()[0])
                f3_ax1.axhline(y=nes[ns], xmin=(xn[ns]-0.4)/15, xmax=(xn[ns]+0.4)/15,linewidth=1, color='k')
                f3_ax1.axvline(x=(xx[ns]-0.4), ymin=lims, ymax=zerlim, linewidth=1, color='k')
                f3_ax1.axvline(x=(xx[ns]+0.4), ymin=lims, ymax=zerlim, linewidth=1, color='k')

        
        

        f3_ax1.axvline(x=4,linewidth=0.3, color='k', linestyle = '-', alpha = 1)
        f3_ax1.axvline(x=9,linewidth=0.3, color='k', linestyle = '-', alpha = 1)
        
        
        f3_ax1.set_xlabel('R             $R_{+}$            $R_{-}$',  fontsize = 7, labelpad=-10, fontstyle = 'italic')
        f3_ax1.xaxis.set_label_position('top')
        #f3_ax2.axhline(y=-5,linewidth=0.2, color='k', linestyle = '-.')
        f3_ax1.axhline(y=0,linewidth=0.3, color='k', linestyle = '-')
        
        if (i == 1) and (j==1):
            f3_ax1.set_title(' '+ region_names[regions[r]], position = (0.5,1.), fontsize = 7)
            
        elif (i == 2) and ((j ==0) or (j==1)):
            
            f3_ax1.set_title(' '+ region_names[regions[r]], position = (0.5,1.), fontsize = 7)
        else:
            f3_ax1.set_title(' '+ region_names[regions[r]], position = (0.5,1.), fontsize = 7)

        r+=1

f3_ax4 = fig3.add_subplot(gs[0:5,10:14])
handles, labels = f3_ax4.get_legend_handles_labels()
f3_ax4.axis('off')





cli_box = mpatches.Circle((0.5,0.5), 1, facecolor="darkblue", label ='Significant Climate contribution $C_{1980}$')
cli2_box = mpatches.Circle((0.5,0.5), 1, facecolor="cornflowerblue", label ='Insignificant Climate contribution $C_{1980}$')
exp_box = mpatches.Circle((0.5,0.5), 1, color='orange', label ='Exposure contribution $E$')
vul_box = mpatches.Circle((0.5,0.5), 1, facecolor='brown', label ='Vulnerability contribution $V$')

imp_box = mpatches.Circle((0.5,0.5), 1, facecolor='mediumpurple', label ='Total Damage Trend (modeled) $M$')
nat_box = mpatches.Rectangle((0, 0), 1, 1, facecolor='white', edgecolor ='k', label ='Total Damage Trend (observed) $N$')



f3_ax4.legend(handles = [cli_box, cli2_box, exp_box,vul_box, imp_box,nat_box], frameon=True, fontsize = 5.5, loc = (-0.15,0.2), edgecolor = 'k')  

plt.savefig('../../data/figures/Figure3.pdf',bbox_inches = 'tight', format = 'pdf')
